#include "gui.h"

GUI::GUI(ServiceTenants &service)
    : service(service)
{
    initGUI();
    connectSignals();
}

void GUI::initGUI()
{
    const QStringList arrayNames = {"number", "surface", "type", "owner"};
    array = new QTableWidget(0, 4);
    array->setHorizontalHeaderLabels(arrayNames);
    secondArray = new QListWidget();
    QVBoxLayout *ly = new QVBoxLayout;
    QWidget *right = new QWidget;
    right->setLayout(ly);

    actionFilterType = new QPushButton("filter type");
    actionFilterSurface = new QPushButton("filter surface");
    actionSortName = new QPushButton("sort name");
    actionSortSurface = new QPushButton("sort surface");
    actionSortTypeSurface = new QPushButton("sort type surface");

    QHBoxLayout *i = new QHBoxLayout;
    QWidget *j = new QWidget;

    i->addWidget(actionFilterType);
    i->addWidget(actionFilterSurface);
    i->addWidget(actionSortName);
    i->addWidget(actionSortSurface);
    i->addWidget(actionSortTypeSurface);

    j->setLayout(i);

    ly->addWidget(array);
    ly->addWidget(secondArray);
    ly->addWidget(j);

    QFormLayout *lyDetails = new QFormLayout;
    number = new QLineEdit;
    lyDetails->addRow(new QLabel("number"), number);
    surface = new QLineEdit;
    lyDetails->addRow(new QLabel("surface"), surface);
    type = new QLineEdit;
    lyDetails->addRow(new QLabel("type"), type);
    owner = new QLineEdit;
    lyDetails->addRow(new QLabel("name"), owner);
    QWidget *details = new QWidget;

    actionAdd = new QPushButton("add tenant");
    actionRemove = new QPushButton("remove tenant");
    actionModify = new QPushButton("modify tenant");
    actionWrite = new QPushButton("write tenants");
    actionSearch = new QPushButton("search tenant");
    actionNotifications = new QPushButton("notifications");
    actionMapReport = new QPushButton("map report");
    actionUndo = new QPushButton("undo");

    lyDetails->addWidget(actionAdd);
    lyDetails->addWidget(actionRemove);
    lyDetails->addWidget(actionModify);
    lyDetails->addWidget(actionWrite);
    lyDetails->addWidget(actionSearch);
    lyDetails->addWidget(actionNotifications);
    lyDetails->addWidget(actionMapReport);
    lyDetails->addWidget(actionUndo);

    details->setLayout(lyDetails);

    newActions = new QWidget;
    newActionsLayout = new QVBoxLayout;
    newActions->setLayout(newActionsLayout);

    lyMain = new QHBoxLayout;
    lyMain->addWidget(details);
    lyMain->addWidget(right);
    lyMain->addWidget(newActions);
    setLayout(lyMain);
}

void GUI::connectSignals()
{
    QObject::connect(actionAdd, &QPushButton::clicked, this, &GUI::addTenant);
    QObject::connect(actionRemove, &QPushButton::clicked, [&]() {
        try
        {
            service.remove(number->text().toInt());
            newArray(service.getTenants());
        }
        catch (ValidException &ex)
        {
            QMessageBox::warning(this, "Warning", QString::fromStdString("valid exception"));
        }
        catch (RepositoryException &ex)
        {
            QMessageBox::warning(this, "Warning", QString::fromStdString("repository exception"));
        }
    });
    QObject::connect(actionModify, &QPushButton::clicked, [&]() {
        try
        {
            service.modify(number->text().toInt(), surface->text().toInt(),
                        type->text().toStdString(), owner->text().toStdString());
            newArray(service.getTenants());
        }
        catch (ValidException &ex)
        {
            QMessageBox::warning(this, "Warning", QString::fromStdString("valid exception"));
        }
        catch (RepositoryException &ex)
        {
            QMessageBox::warning(this, "Warning", QString::fromStdString("repository exception"));
        }
    });
    QObject::connect(actionWrite, &QPushButton::clicked, [&]() {
        try
        {
            newArray(service.getTenants());
        }
        catch (ValidException &ex)
        {
            QMessageBox::warning(this, "Warning", QString::fromStdString("valid exception"));
        }
        catch (RepositoryException &ex)
        {
            QMessageBox::warning(this, "Warning", QString::fromStdString("repository exception"));
        }
    });
    QObject::connect(actionSearch, &QPushButton::clicked, [&]() {
        try
        {
            newArray(service.search(number->text().toInt()));
        }
        catch (ValidException &ex)
        {
            QMessageBox::warning(this, "Warning", QString::fromStdString("valid exception"));
        }
        catch (RepositoryException &ex)
        {
            QMessageBox::warning(this, "Warning", QString::fromStdString("repository exception"));
        }
    });
    QObject::connect(actionFilterType, &QPushButton::clicked, [&]() {
        try
        {
            newArray(service.filterType(type->text().toStdString()));
        }
        catch (ValidException &ex)
        {
            QMessageBox::warning(this, "Warning", QString::fromStdString("valid exception"));
        }
        catch (RepositoryException &ex)
        {
            QMessageBox::warning(this, "Warning", QString::fromStdString("repository exception"));
        }
    });
    QObject::connect(actionFilterSurface, &QPushButton::clicked, [&]() {
        try
        {
            newArray(service.filterSurface(surface->text().toInt()));
        }
        catch (ValidException &ex)
        {
            QMessageBox::warning(this, "Warning", QString::fromStdString("valid exception"));
        }
        catch (RepositoryException &ex)
        {
            QMessageBox::warning(this, "Warning", QString::fromStdString("repository exception"));
        }
    });
    QObject::connect(actionSortName, &QPushButton::clicked, [&]() {
        newArray(service.sortName());
    });
    QObject::connect(actionSortSurface, &QPushButton::clicked, [&]() {
        newArray(service.sortSurface());
    });
    QObject::connect(actionSortTypeSurface, &QPushButton::clicked, [&]() {
        newArray(service.sortTypeSurface());
    });
    QObject::connect(actionNotifications, &QPushButton::clicked, [&]() {
        secondGUI *notifications = new secondGUI(service);
        notifications->show();
    });
    QObject::connect(actionMapReport, &QPushButton::clicked, [&]() {
        try
        {
            array->setRowCount(0);
            auto *i = newActionsLayout->takeAt(0);
            while (i)
            {
                delete i->widget();
                delete i;
                i = newActionsLayout->takeAt(0);
            }
            for (const auto& [surface, number] : service.mapReport())
            {
                array->insertRow(array->rowCount());
                array->setItem(array->rowCount() - 1, 0, new QTableWidgetItem(QString::number(number)));
                array->setItem(array->rowCount() - 1, 1, new QTableWidgetItem(QString::number(surface)));
                array->setItem(array->rowCount() - 1, 2, new QTableWidgetItem(QString::fromStdString("")));
                array->setItem(array->rowCount() - 1, 3, new QTableWidgetItem(QString::fromStdString("")));
                auto pointer = new QPushButton(QString::number(surface));
                newActionsLayout->addWidget(pointer);
                QObject::connect(pointer, &QPushButton::clicked, [number]() {
                    QMessageBox::information(nullptr, "number of apartments", QString::number(number));
                });
            }
        }
        catch (ValidException &ex)
        {
            QMessageBox::warning(this, "Warning", QString::fromStdString("valid exception"));
        }
        catch (RepositoryException &ex)
        {
            QMessageBox::warning(this, "Warning", QString::fromStdString("repository exception"));
        }
    });
    QObject::connect(actionUndo, &QPushButton::clicked, [&]() {
        try
        {
            service.undo();
            newArray(service.getTenants());
        }
        catch (ValidException &ex)
        {
            QMessageBox::warning(this, "Warning", QString::fromStdString("valid exception"));
        }
        catch (RepositoryException &ex)
        {
            QMessageBox::warning(this, "Warning", QString::fromStdString("repository exception"));
        }
    });
}

void GUI::newArray(const std::vector<Tenant>& tenants)
{
    array->setRowCount(0);
    for (const auto& tenant : tenants)
    {
        array->insertRow(array->rowCount());
        array->setItem(array->rowCount() - 1, 0, new QTableWidgetItem(QString::number(tenant.getNumber())));
        array->setItem(array->rowCount() - 1, 1, new QTableWidgetItem(QString::number(tenant.getSurface())));
        array->setItem(array->rowCount() - 1, 2, new QTableWidgetItem(QString::fromStdString(tenant.getType())));
        array->setItem(array->rowCount() - 1, 3, new QTableWidgetItem(QString::fromStdString(tenant.getName())));
    }

    secondArray->clear();
    for (const auto& tenant : tenants)
        secondArray->addItem(QString::number(tenant.getNumber()));
}

void GUI::newArray(const Tenant& tenant)
{
    array->setRowCount(0);
    array->insertRow(array->rowCount());
    array->setItem(array->rowCount() - 1, 0, new QTableWidgetItem(QString::number(tenant.getNumber())));
    array->setItem(array->rowCount() - 1, 1, new QTableWidgetItem(QString::number(tenant.getSurface())));
    array->setItem(array->rowCount() - 1, 2, new QTableWidgetItem(QString::fromStdString(tenant.getType())));
    array->setItem(array->rowCount() - 1, 3, new QTableWidgetItem(QString::fromStdString(tenant.getName())));

    secondArray->clear();
    secondArray->addItem(QString::number(tenant.getNumber()));
}

void GUI::addTenant()
{
    try
    {
        service.add(number->text().toInt(), surface->text().toInt(),
                    type->text().toStdString(), owner->text().toStdString());
        newArray(service.getTenants());
    }
    catch (ValidException &ex)
    {
        QMessageBox::warning(this, "Warning", QString::fromStdString("valid exception"));
    }
    catch (RepositoryException &ex)
    {
        QMessageBox::warning(this, "Warning", QString::fromStdString("repository exception"));
    }
}

secondGUI::secondGUI(ServiceTenants &service)
    : service(service)
{
    initGUI();
    connectSignals();
}

void secondGUI::initGUI()
{
    const QStringList arrayNames = {"number", "surface", "type", "owner"};
    array = new QTableWidget(0, 4);
    array->setHorizontalHeaderLabels(arrayNames);
    QVBoxLayout *ly = new QVBoxLayout;
    QWidget *right = new QWidget;
    right->setLayout(ly);

    actionAddNotification = new QPushButton("add notification");
    actionClearNotifications = new QPushButton("clear notifications");
    actionRandomNotification = new QPushButton("random notification");
    actionWriteNotifications = new QPushButton("write notifications");
    actionExportHTML = new QPushButton("export HTML");
    actionExportCSV = new QPushButton("export CSV");

    QHBoxLayout *i = new QHBoxLayout;
    QWidget *j = new QWidget;


    i->addWidget(actionAddNotification);
    i->addWidget(actionClearNotifications);
    i->addWidget(actionRandomNotification);
    i->addWidget(actionWriteNotifications);
    j->setLayout(i);

    ly->addWidget(array);
    ly->addWidget(j);

    QFormLayout *lyDetails = new QFormLayout;
    number = new QLineEdit;
    lyDetails->addRow(new QLabel("number"), number);
    owner = new QLineEdit;
    lyDetails->addRow(new QLabel("name"), owner);
    QWidget *details = new QWidget;

    lyDetails->addWidget(actionExportHTML);
    lyDetails->addWidget(actionExportCSV);
    details->setLayout(lyDetails);

    lyMain = new QHBoxLayout;
    lyMain->addWidget(details);
    lyMain->addWidget(right);
    setLayout(lyMain);
}

void secondGUI::connectSignals()
{
    QObject::connect(actionAddNotification, &QPushButton::clicked, [&]() {
        try
        {
            service.addCart(number->text().toInt());
            newArray(service.getCart());
        }
        catch (ValidException &ex)
        {
            QMessageBox::warning(this, "Warning", QString::fromStdString("valid exception"));
        }
        catch (RepositoryException &ex)
        {
            QMessageBox::warning(this, "Warning", QString::fromStdString("repository exception"));
        }
    });
    QObject::connect(actionClearNotifications, &QPushButton::clicked, [&]() {
        try
        {
            service.clearCart();
            newArray(service.getCart());
        }
        catch (ValidException &ex)
        {
            QMessageBox::warning(this, "Warning", QString::fromStdString("valid exception"));
        }
        catch (RepositoryException &ex)
        {
            QMessageBox::warning(this, "Warning", QString::fromStdString("repository exception"));
        }
    });
    QObject::connect(actionRandomNotification, &QPushButton::clicked, [&]() {
        try
        {
            service.randomCart(number->text().toInt());
            newArray(service.getCart());
        }
        catch (ValidException &ex)
        {
            QMessageBox::warning(this, "Warning", QString::fromStdString("valid exception"));
        }
        catch (RepositoryException &ex)
        {
            QMessageBox::warning(this, "Warning", QString::fromStdString("repository exception"));
        }
    });
    QObject::connect(actionWriteNotifications, &QPushButton::clicked, [&]() {
        try
        {
            newArray(service.getCart());
        }
        catch (ValidException &ex)
        {
            QMessageBox::warning(this, "Warning", QString::fromStdString("valid exception"));
        }
        catch (RepositoryException &ex)
        {
            QMessageBox::warning(this, "Warning", QString::fromStdString("repository exception"));
        }
    });
    QObject::connect(actionExportHTML, &QPushButton::clicked, [&]() {
        try
        {
            service.exportHTML(owner->text().toStdString());
        }
        catch (ValidException &ex)
        {
            QMessageBox::warning(this, "Warning", QString::fromStdString("valid exception"));
        }
        catch (RepositoryException &ex)
        {
            QMessageBox::warning(this, "Warning", QString::fromStdString("repository exception"));
        }
    });
    QObject::connect(actionExportCSV, &QPushButton::clicked, [&]() {
        try
        {
            service.exportCSV(owner->text().toStdString());
        }
        catch (ValidException &ex)
        {
            QMessageBox::warning(this, "Warning", QString::fromStdString("valid exception"));
        }
        catch (RepositoryException &ex)
        {
            QMessageBox::warning(this, "Warning", QString::fromStdString("repository exception"));
        }
    });
}

void secondGUI::newArray(const std::vector<Tenant>& tenants)
{
    array->setRowCount(0);
    for (const auto& tenant : tenants)
    {
        array->insertRow(array->rowCount());
        array->setItem(array->rowCount() - 1, 0, new QTableWidgetItem(QString::number(tenant.getNumber())));
        array->setItem(array->rowCount() - 1, 1, new QTableWidgetItem(QString::number(tenant.getSurface())));
        array->setItem(array->rowCount() - 1, 2, new QTableWidgetItem(QString::fromStdString(tenant.getType())));
        array->setItem(array->rowCount() - 1, 3, new QTableWidgetItem(QString::fromStdString(tenant.getName())));
    }
}

void secondGUI::newArray(const Tenant& tenant)
{
    array->setRowCount(0);
    array->insertRow(array->rowCount());
    array->setItem(array->rowCount() - 1, 0, new QTableWidgetItem(QString::number(tenant.getNumber())));
    array->setItem(array->rowCount() - 1, 1, new QTableWidgetItem(QString::number(tenant.getSurface())));
    array->setItem(array->rowCount() - 1, 2, new QTableWidgetItem(QString::fromStdString(tenant.getType())));
    array->setItem(array->rowCount() - 1, 3, new QTableWidgetItem(QString::fromStdString(tenant.getName())));
}
