#include "repository.h"

// adds tenant to repository
void Repository::add(const Tenant& obj)
{
	if (exists(obj.getNumber()))
	{
		std::vector<std::string> s;
		s.push_back("numarul dat exita deja");
		throw RepositoryException(s);
	}
	archive.push_back(obj);
}

// search if a number exists
bool Repository::exists(const int& number)
{
	return std::find_if(archive.begin(), archive.end(),
		[number](const Tenant& tenant)
		{
			return tenant.getNumber() == number;
		}
	) != std::end(archive);
}

// removes tenant from repository using id
void Repository::remove(const int& number)
{
	auto i = std::find_if(archive.begin(), archive.end(),
		[number](const Tenant& tenant)
		{
			return tenant.getNumber() == number;
		}
	);
	if (i != std::end(archive))
	{
		archive.erase(i);
		return;
	}
	std::vector<std::string> s;
	s.push_back("nu exista");
	throw RepositoryException(s);
}

// searches tenant using id
const Tenant& Repository::search(const int& number)
{
	auto i = std::find_if(archive.begin(), archive.end(),
		[number](const Tenant& tenant)
		{
			return tenant.getNumber() == number;
		}
	);
	if (i != std::end(archive))
		return *i;
	std::vector<std::string> s;
	s.push_back("nu exista");
	throw RepositoryException(s);
}

// returns all tenants as std::vector
std::vector<Tenant> Repository::getTenants() const
{
	return archive;
}

RepositoryMap::RepositoryMap(const long double& p)
: p(p)
{
}

// adds tenant to repository
void RepositoryMap::add(const Tenant& obj)
{
	std::srand(std::time(nullptr));
	long double n = std::rand() % 10;
	n /= 10;
	if (p == 1 || p > n)
	{
		std::vector<std::string> s;
		s.push_back("random number");
		throw RepositoryException(s);
	}
	std::vector<std::string> s;
	s.push_back("numarul dat exita deja");
	if (archive.find(obj.getNumber()) != archive.end())
		throw RepositoryException(s);
	archive.insert({obj.getNumber(), obj});
}

// search if a number exists
bool RepositoryMap::exists(const int& number)
{
	std::srand(std::time(nullptr));
	long double n = std::rand() % 10;
	n /= 10;
	if (p == 1 || p > n)
	{
		std::vector<std::string> s;
		s.push_back("random number");
		throw RepositoryException(s);
	}
	
	return archive.find(number) != archive.end();
}

// removes tenant from repository using id
void RepositoryMap::remove(const int& number)
{
	std::srand(std::time(nullptr));
	long double n = std::rand() % 10;
	n /= 10;
	if (p == 1 || p > n)
	{
		std::vector<std::string> s;
		s.push_back("random number");
		throw RepositoryException(s);
	}
	
	std::vector<std::string> s;
	s.push_back("nu exista");
	if (archive.find(number) == archive.end())
		throw RepositoryException(s);
	archive.erase(number);
}

// searches tenant using id
const Tenant& RepositoryMap::search(const int& number)
{
	std::srand(std::time(nullptr));
	long double n = std::rand() % 10;
	n /= 10;
	if (p == 1 || p > n)
	{
		std::vector<std::string> s;
		s.push_back("random number");
		throw RepositoryException(s);
	}
	
	for (auto i = archive.begin(); i != archive.end(); ++i)
		if (i->second.getNumber() == number)
			return i->second;
	std::vector<std::string> s;
	s.push_back("nu exista");
	throw RepositoryException(s);
}

// returns all tenants as std::vector
std::vector<Tenant> RepositoryMap::getTenants() const
{
	std::srand(std::time(nullptr));
	long double n = std::rand() % 10;
	n /= 10;
	if (p == 1 || p > n)
	{
		std::vector<std::string> s;
		s.push_back("random number");
		throw RepositoryException(s);
	}
	
	std::vector<Tenant> tenants;
	for (auto i = archive.begin(); i != archive.end(); ++i)
		tenants.push_back(i->second);
	return tenants;
}

long double& RepositoryMap::probability()
{
	return p;
}

// constructor for repository exception
RepositoryException::RepositoryException(const std::vector<std::string>& s)
: s(s)
{
}

// << operator overload
std::ostream& operator<<(std::ostream& out, const RepositoryException& e)
{
	for (const auto& i : e.s)
		out << i << '\n';
	return out;
}