#ifndef REPOSITORY_H
#define REPOSITORY_H

#include <vector>
#include "domain.h"
#include <string>
#include <sstream>
#include <algorithm>
#include <map>
#include <cstdlib>
#include <ctime>

class RepositoryAbstract
{

public:

	virtual void add(const Tenant&) = 0;
	virtual void remove(const int&) = 0;
	virtual bool exists(const int&) = 0;
	virtual const Tenant& search(const int&) = 0;
	virtual std::vector<Tenant> getTenants() const = 0;
	
};

class Repository : public RepositoryAbstract
{
	
public:
	
	void add(const Tenant&) override;
	void remove(const int&) override;
	bool exists(const int&) override;
	const Tenant& search(const int&) override;
	std::vector<Tenant> getTenants() const override;
	
private:

	std::vector<Tenant> archive;
	
};

class RepositoryMap : public RepositoryAbstract
{
	
public:
	
	RepositoryMap(const long double&);
	void add(const Tenant&) override;
	void remove(const int&) override;
	bool exists(const int&) override;
	const Tenant& search(const int&) override;
	std::vector<Tenant> getTenants() const override;
	long double& probability();
	
private:

	long double p;
	std::map<int, Tenant> archive;
	
};

class RepositoryException
{
public:
	RepositoryException(const std::vector<std::string>& s);
	friend std::ostream& operator<<(std::ostream& out, const RepositoryException& e);
private:
	std::vector<std::string> s;
};

#endif