#ifndef SERVICE_H
#define SERVICE_H

#include "repository.h"
#include "domain.h"
#include "valid.h"
#include <vector>
#include <algorithm>
#include <ctime>
#include <cstdlib>
#include <map>
#include "undo.h"
#include "cart.h"
#include "export.h"

class ServiceTenants
{
	
public:

	ServiceTenants(RepositoryAbstract&, Valid);
	~ServiceTenants();
	void add(int, int, std::string, std::string);
	void remove(int);	
	void modify(int, int, std::string, std::string);
	const Tenant& search(int);
	std::vector<Tenant> getTenants();
	std::vector<Tenant> filterType(const std::string&);
	std::vector<Tenant> filterSurface(const int& surface);
	std::vector<Tenant> sortName();
	std::vector<Tenant> sortSurface();
	std::vector<Tenant> sortTypeSurface();

	void addCart(const unsigned int&);
	void clearCart();
	const std::vector<Tenant>& getCart() const;
	void randomCart(const unsigned int&);

	std::map<int, int> mapReport();
	
	void exportHTML(const std::string&);
	void exportCSV(const std::string&);
	
	void undo();
	
private:

	RepositoryAbstract& repository;
	Valid valid;
	
	Cart cart;
	
	std::vector<UndoAction*> undoStack;
	
};

#endif