#include "test.h"

// test domain
void testDomain()
{
	Tenant t(1, 2, "3", "4");
	assert(t.getNumber() == 1);
	assert(t.getSurface() == 2);
	assert(t.getType() == "3");
	assert(t.getName() == "4");
	
	Tenant t2;
	assert(t2.getNumber() == 0);
	assert(t2.getSurface() == 0);
	assert(t2.getType() == "");
	assert(t2.getName() == "");
}

// test repository vector
void testRepositoryVector()
{
	Repository r;
	Tenant t1(1, 2, "3", "4"), t2(2, 3, "4", "5"), t3;
	
	r.add(t1);
	r.add(t2);
	
	try
	{
		r.add(t1);
		assert(0);
	}
	catch(const RepositoryException& ex)
	{
		assert(1);
	}
	
	std::vector<Tenant> t = r.getTenants();
	
	assert(t[0].getNumber() == 1);
	assert(t[0].getSurface() == 2);
	assert(t[0].getType() == "3");
	assert(t[0].getName() == "4");
	
	assert(t[1].getNumber() == 2);
	assert(t[1].getSurface() == 3);
	assert(t[1].getType() == "4");
	assert(t[1].getName() == "5");
	
	try
	{
		r.remove(3);
		assert(0);
	}
	catch(const RepositoryException& ex)
	{
		assert(1);
	}
	
	assert(r.exists(1));
	assert(r.exists(2));
	assert(!r.exists(3));
	
	try
	{
		t3 = r.search(1);
		
		assert(t3.getNumber() == 1);
		assert(t3.getSurface() == 2);
		assert(t3.getType() == "3");
		assert(t3.getName() == "4");
	}
	catch(const RepositoryException& ex)
	{
		assert(0);
	}
	
	try
	{
		t3 = r.search(5);
		assert(0);
	}
	catch(const RepositoryException& ex)
	{
		std::stringstream out;
		out << ex;
		assert(out.str() == "nu exista\n");
	}
	
	r.remove(1);
	
	t = r.getTenants();
	
	assert(t[0].getNumber() == 2);
	assert(t[0].getSurface() == 3);
	assert(t[0].getType() == "4");
	assert(t[0].getName() == "5");
}

// test repository map
void testRepositoryMap()
{
	RepositoryMap r(0);
	Tenant t1(1, 2, "3", "4"), t2(2, 3, "4", "5"), t3;
	
	r.add(t1);
	r.add(t2);
	
	try
	{
		r.add(t1);
		assert(0);
	}
	catch(const RepositoryException& ex)
	{
		assert(1);
	}
	
	std::vector<Tenant> t = r.getTenants();
	
	assert(t[0].getNumber() == 1);
	assert(t[0].getSurface() == 2);
	assert(t[0].getType() == "3");
	assert(t[0].getName() == "4");
	
	assert(t[1].getNumber() == 2);
	assert(t[1].getSurface() == 3);
	assert(t[1].getType() == "4");
	assert(t[1].getName() == "5");
	
	try
	{
		r.remove(3);
		assert(0);
	}
	catch(const RepositoryException& ex)
	{
		assert(1);
	}
	
	assert(r.exists(1));
	assert(r.exists(2));
	assert(!r.exists(3));
	
	try
	{
		t3 = r.search(1);
		
		assert(t3.getNumber() == 1);
		assert(t3.getSurface() == 2);
		assert(t3.getType() == "3");
		assert(t3.getName() == "4");
	}
	catch(const RepositoryException& ex)
	{
		assert(0);
	}
	
	try
	{
		t3 = r.search(5);
		assert(0);
	}
	catch(const RepositoryException& ex)
	{
		std::stringstream out;
		out << ex;
		assert(out.str() == "nu exista\n");
	}
	
	r.remove(1);
	
	t = r.getTenants();
	
	assert(t[0].getNumber() == 2);
	assert(t[0].getSurface() == 3);
	assert(t[0].getType() == "4");
	assert(t[0].getName() == "5");
	
	RepositoryMap rMap(0);
	assert(rMap.probability() == 0);
	rMap.probability() = 1;
	assert(rMap.probability() == 1);
	
	try
	{
		rMap.add(t1);
		assert(0);
	}
	catch(const RepositoryException& ex)
	{
		assert(1);
	}
	
	rMap.probability() = 0;
	rMap.add(t1);
	rMap.probability() = 1;
	
	try
	{
		rMap.exists(1);
		assert(0);
	}
	catch(const RepositoryException& ex)
	{
		assert(1);
	}
	
	try
	{
		rMap.remove(1);
		assert(0);
	}
	catch(const RepositoryException& ex)
	{
		assert(1);
	}
	
	try
	{
		rMap.search(1);
		assert(0);
	}
	catch(const RepositoryException& ex)
	{
		assert(1);
	}
	
	try
	{
		rMap.getTenants();
		assert(0);
	}
	catch(const RepositoryException& ex)
	{
		assert(1);
	}
}

// test repository
void testRepository()
{
	testRepositoryVector();
	testRepositoryMap();
}

// test service
void testService()
{
	Repository r;
	Valid v;
	ServiceTenants s(r, v);
	
	assert(s.getTenants().size() == 0);
	s.add(1, 3, "33333", "44444");
	assert(s.getTenants().size() == 1);
	s.add(2, 1, "44444", "55555");
	assert(s.getTenants().size() == 2);
	s.add(3, 2, "44444", "11111");
	s.add(4, 1, "55555", "55555");
	
	for (int i = 0; i < 5; ++i)
		s.undo();
	
	assert(s.getTenants().size() == 0);
	
	s.add(1, 3, "33333", "44444");
	s.add(2, 1, "44444", "55555");
	s.add(3, 2, "44444", "11111");
	s.add(4, 1, "55555", "55555");
	
	s.addCart(1);
	assert(s.getCart().size() == 1);
	s.clearCart();
	assert(s.getCart().size() == 0);
	s.randomCart(3);
	assert(s.getCart().size() == 3);
	
	int c = 0;
	for (const auto& [surface, number] : s.mapReport())
	{
		if (surface == 1)
			assert (number == 2);
		else
			assert(number == 1);
		
		++c;
		
		assert (surface == c);
	}
	
	s.remove(4);
	
	try
	{
		s.add(-1, -1, "", "1");
		assert(0);
	}
	catch(const ValidException& ex)
	{
		assert(1);
	}
	
	std::vector<Tenant> t2 = s.filterType("33333");
	assert(t2.size() == 1);
	
	t2 = s.filterSurface(3);
	assert(t2.size() == 1);
	
	t2 = s.sortName();
	assert (t2[0].getNumber() == 3);
	
	t2 = s.sortSurface();
	assert (t2[0].getNumber() == 2);
	
	t2 = s.sortTypeSurface();
	assert (t2[0].getNumber() == 1);
	
	s.remove(1);
	assert(s.getTenants().size() == 2);
	
	s.modify(2, 2, "33333", "55555");
	
	Tenant t = s.search(2);
	
	assert(t.getNumber() == 2);
	assert(t.getSurface() == 2);
	assert(t.getType() == "33333");
	assert(t.getName() == "55555");
}

// test validation
void testValid()
{
	Valid v;
	Tenant t(-1, 1, "3", "5");
	
	try
	{
		v.validate(t);
	}
	catch (const ValidException& ex)
	{
		std::stringstream out;
		out << ex;
		auto s = out.str();
		assert(s.find("error tenant.getNumber() <= 0") >= 0);
		assert(s.find("error tenant.getType() < 5") >= 0);
		assert(s.find("error tenant.getName() < 5") >= 0);
	}
}

// test cart
void testCart()
{
	Cart cart;
	
	Tenant t1(1, 2, "3", "4");
	Tenant t2(2, 3, "4", "5");
	
	cart.add(t1);
	assert(cart.get().size() == 1);
	
	cart.add(t2);
	assert(cart.get().size() == 2);
	
	cart.clear();
	assert(cart.get().size() == 0);
	
	std::vector<Tenant> tenants;
	tenants.push_back(t1);
	tenants.push_back(t2);
	
	cart.random(1, tenants);
	assert(cart.get().size() == 1);
	
	cart.clear();
	
	cart.random(2, tenants);
	assert(cart.get().size() == 2);
}

// test eHTML
void testHTML()
{
	Repository r;
	Valid v;
	ServiceTenants service(r, v);
	
	service.add(1, 2, "33333", "44444");
	service.add(2, 3, "44444", "55555");
	
	service.addCart(1);
	service.addCart(2);
	
	service.exportHTML("test.html");
	
	std::ifstream fin("test.html");
	
	assert(fin.is_open());
	
	int c = 0;
	std::string s;
	
	while (!fin.eof())
	{
		std::getline(fin, s);
		++c;
	}
	
	fin.close();
	
	assert(c == 17);
	
	try
	{
		service.exportHTML("test/test.html");
		assert(0);
	}
	catch (std::runtime_error&)
	{
		assert(1);
	}
}

// test eCSV
void testCSV()
{
	Repository r;
	Valid v;
	ServiceTenants service(r, v);
	
	service.add(1, 2, "33333", "44444");
	service.add(2, 3, "44444", "55555");
	
	service.addCart(1);
	service.addCart(2);
	service.exportCSV("test.out");
	
	std::ifstream fin("test.out");
	
	assert(fin.is_open());
	
	int c = 0;
	std::string s;
	
	while (!fin.eof())
	{
		std::getline(fin, s);
		++c;
	}
	
	fin.close();
	
	assert(c == 3);
	
	try
	{
		service.exportCSV("test/test.out");
		assert(0);
	}
	catch (std::runtime_error&)
	{
		assert(1);
	}
}

// test export
void testExport()
{
	testHTML();
	testCSV();
}

// test undo
void testUndo()
{
	Repository repository;
	Valid valid;
	ServiceTenants service(repository, valid);
	
	service.add(1, 2, "33333", "44444");
	service.add(2, 3, "44444", "55555");
	
	service.undo();
	assert(service.getTenants().size() == 1);
	
	service.undo();
	assert(service.getTenants().size() == 0);
	
	service.add(1, 2, "33333", "44444");
	service.add(2, 3, "44444", "55555");
	
	service.remove(1);
	service.undo();
	assert(service.getTenants().size() == 2);
	
	service.modify(2, 5, "55555", "55555");
	service.undo();
	assert(service.getTenants()[1].getSurface() == 3);
}

// test all
void test()
{
	testDomain();
	testRepository();
	testService();
	testValid();
	testCart();
	testExport();
	testUndo();
}